﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using SimonPG.WaitCore; // ←[Wait機能を使用したいところでこのコードを追加]

namespace EventWaitingCore {

    public partial class TestForm : Form {

        public TestForm() {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e) {
            Console.WriteLine("button1がクリックされました。");
        }

        private void button2_Click(object sender, EventArgs e) {
            Console.WriteLine("button2がクリックされました。");
        }

        EventWaiter _Waiter;
        private void _UITest_Cancel_Click(object sender, EventArgs e) {
            _Waiter.CancelWait();
        }

        private void _UITest_0_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1をクリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;

            var waiter = new EventWaiter(_Test_0_Handler);

            waiter.Finally += _FinallyHandler;
        }

        void waiter_Finally(object sender, EventArgs e) {
            throw new NotImplementedException();
        }

        IEnumerable<bool> _Test_0_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new Event(button1, "Click"));
            Console.WriteLine("...処理B...");
        }

        private void _UITest_1_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1とbutton2をクリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;
            button2.Enabled = true;

            var waiter = new EventWaiter(_Test_1_Handler);

            waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_1_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new And(new Event(button1, "Click"), new Event(button2, "Click")));
            Console.WriteLine("...処理B...");
        }

        private void _UITest_2_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1は押さずにbutton2をクリックしてください。");
            Console.WriteLine("button1を押した場合は無限に待機してしまうのでアプリケーションを再起動します。\n");

            _SetEnableForTest();
            button1.Enabled = true;
            button2.Enabled = true;

            var waiter = new EventWaiter(_Test_2_Handler);

            waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_2_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new And(
                new Not(new Event(button1, "Click")), new Event(button2, "Click")));
            Console.WriteLine("...処理B...");
        }

        private void _UITest_3_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1を3回クリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;

            var waiter = new EventWaiter(_Test_3_Handler);

            waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_3_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new Count(new Event(button1, "Click"), 3));
            Console.WriteLine("...処理B...");
        }

        private void _UITest_4_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1をクリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;

            var waiter = new EventWaiter(_Test_4_Handler);

            waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_4_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(
                new Eval(new Event(button1, "Click"), () => button1.Text == "button1"));
            Console.WriteLine("...処理B...");
        }

        private void _UITest_5_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("5秒後にbutton1をクリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;

            var waiter = new EventWaiter(_Test_5_Handler);

            waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_5_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new SecondAfter(new Event(button1, "Click"), 5));
            Console.WriteLine("...処理B...");
        }

        public class SecondAfter : Condition {

            public double Second;
            public EventContainer Container { get { return _Containers.First(); } }
            bool _IsStarted = false;
            int _StartTick = 0;

            public SecondAfter(EventContainer container, double second)
                : base(container) { this.Second = second; }

            protected override void OnSetupWait(ActionWrapper on_end_wait) {
                _IsStarted = true;
                _StartTick = Environment.TickCount;
                on_end_wait.Val += () => { _IsStarted = false; };
            }

            protected override bool OnUpdateEvaluation(ActionWrapper on_end_update) {

                if (_IsStarted) {

                    var elapsed = Environment.TickCount - _StartTick;
                    var thres = (int)(this.Second * 1e3);
                    if (elapsed >= thres)
                        return true;
                }

                if (this.Container.Evaluation) {
                    on_end_update.Val += this.Container.Reset;
                }
                return false;
            }
        }

        private void _UITest_6_Click(object sender, EventArgs e) {

            Console.WriteLine("\n------------------------------");
            Console.WriteLine("テスト開始: \"{0}\"", ((Control)sender).Text);
            Console.WriteLine("button1は押さずにキャンセルをクリックしてください。\n");

            _SetEnableForTest();
            button1.Enabled = true;
            _UITest_Cancel.Enabled = true;

            _Waiter = new EventWaiter(_Test_6_Handler);

            _Waiter.Finally += _FinallyHandler;
        }

        IEnumerable<bool> _Test_6_Handler(EventWaiter waiter) {

            Console.WriteLine("...処理A...");
            yield return waiter.Wait(new Event(button1, "Click"));
            Console.WriteLine("...処理B...");
        }

        void _SetEnableForTest() {
            button1.Enabled = false;
            button2.Enabled = false;
            _UITest_Cancel.Enabled = false;
            _UITest_0.Enabled = false;
            _UITest_1.Enabled = false;
            _UITest_2.Enabled = false;
            _UITest_3.Enabled = false;
            _UITest_4.Enabled = false;
            _UITest_5.Enabled = false;
            _UITest_6.Enabled = false;
        }
        void _ResetEnable() {
            button1.Enabled = false;
            button2.Enabled = false;
            _UITest_Cancel.Enabled = false;
            _UITest_0.Enabled = true;
            _UITest_1.Enabled = true;
            _UITest_2.Enabled = true;
            _UITest_3.Enabled = true;
            _UITest_4.Enabled = true;
            _UITest_5.Enabled = true;
            _UITest_6.Enabled = true;
        }
        void _FinallyHandler(object sender, EventArgs e) {
            _ResetEnable();
            Console.WriteLine("\nテスト完了");
        }

        private void TestForm_Load(object sender, EventArgs e) {

            _UIExtra_TestTree.Nodes.AddRange(new[] { 
                new TreeNode("Eventクラスの確認", new[] {
                    new ActionTreeNode("タイマイベントを待機します", 
                        new Test_EventWaiting().Test_WaitUsingYield),
                }),
                new TreeNode("And/Orクラスの確認", new[] {
                    new ActionTreeNode("単純に3つのタイマイベントをAnd/Or結合して待機します", 
                        new Test_AndOrWaiting().Test_AndOrConnection),
                    new ActionTreeNode("複雑にAnd/Or結合して待機します", 
                        new Test_AndOrWaiting().Test_AdvancedConnection),
                }),
                new TreeNode("Notクラスの確認", new[] {
                    new ActionTreeNode("タイマイベントをNot結合して待機します", 
                        new Test_NotWaiting().Test_NotConnection),
                    new ActionTreeNode("複雑にNot結合して待機します", 
                        new Test_NotWaiting().Test_AdvancedConnection),
                }),
                new TreeNode("EventWaiterクラスの確認", new[] {
                    new ActionTreeNode("CancelWait関数で待機をキャンセルします",
                        new Test_Canceling().Test_CancelWait),
                    new ActionTreeNode("CancelFunc関数で待機全体を途中で終了します", 
                        new Test_Canceling().Test_CancelFunc),
                }),
                new TreeNode("Countクラスの確認", new[] {
                    new ActionTreeNode("タイマイベントが複数回発生するまで待機します", 
                        new Test_UserWaiting().Test_CountWaiting),
                }),
                new TreeNode("Evalクラスの確認", new[] {
                    new ActionTreeNode("true/falseの条件文をイベントと合わせて待機します", 
                        new Test_EvalWaiting().Test_WaitWithEval),
                }),
            });
            _UIExtra_TestTree.ExpandAll();
        }

        class ActionTreeNode : TreeNode {
            Action _Action;
            public ActionTreeNode(string text, Action action)
                : base(text) { _Action = action; }
            public void TakeAction() { _Action(); }
        }

        private void _UIExtra_TestTree_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e) {

            var test_action = e.Node as ActionTreeNode;
            if (test_action == null)
                return;

            test_action.TakeAction();
        }
    }
}
